from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.layout import Layout
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class LegendEntry(Serialisable):
    tagname: str
    idx: Incomplete
    delete: Incomplete
    txPr: Typed[RichText, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, idx: int = 0, delete: bool = False, txPr: RichText | None = None, extLst: Unused = None) -> None: ...

class Legend(Serialisable):
    tagname: str
    legendPos: Incomplete
    position: Alias
    legendEntry: Incomplete
    layout: Typed[Layout, Literal[True]]
    overlay: Incomplete
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    txPr: Typed[RichText, Literal[True]]
    textProperties: Alias
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        legendPos: str = "r",
        legendEntry=(),
        layout: Layout | None = None,
        overlay: Incomplete | None = None,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        extLst: Unused = None,
    ) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.descriptors import Strict, String
from openpyxl.descriptors.base import Integer, _ConvertibleToInt
from openpyxl.descriptors.serialisable import Serialisable

BUILTIN_FORMATS: Incomplete
BUILTIN_FORMATS_MAX_SIZE: int
BUILTIN_FORMATS_REVERSE: Incomplete
FORMAT_GENERAL: Incomplete
FORMAT_TEXT: Incomplete
FORMAT_NUMBER: Incomplete
FORMAT_NUMBER_00: Incomplete
FORMAT_NUMBER_COMMA_SEPARATED1: Incomplete
FORMAT_NUMBER_COMMA_SEPARATED2: str
FORMAT_PERCENTAGE: Incomplete
FORMAT_PERCENTAGE_00: Incomplete
FORMAT_DATE_YYYYMMDD2: str
FORMAT_DATE_YYMMDD: str
FORMAT_DATE_DDMMYY: str
FORMAT_DATE_DMYSLASH: str
FORMAT_DATE_DMYMINUS: str
FORMAT_DATE_DMMINUS: str
FORMAT_DATE_MYMINUS: str
FORMAT_DATE_XLSX14: Incomplete
FORMAT_DATE_XLSX15: Incomplete
FORMAT_DATE_XLSX16: Incomplete
FORMAT_DATE_XLSX17: Incomplete
FORMAT_DATE_XLSX22: Incomplete
FORMAT_DATE_DATETIME: str
FORMAT_DATE_TIME1: Incomplete
FORMAT_DATE_TIME2: Incomplete
FORMAT_DATE_TIME3: Incomplete
FORMAT_DATE_TIME4: Incomplete
FORMAT_DATE_TIME5: Incomplete
FORMAT_DATE_TIME6: Incomplete
FORMAT_DATE_TIME7: str
FORMAT_DATE_TIME8: str
FORMAT_DATE_TIMEDELTA: str
FORMAT_DATE_YYMMDDSLASH: str
FORMAT_CURRENCY_USD_SIMPLE: str
FORMAT_CURRENCY_USD: str
FORMAT_CURRENCY_EUR_SIMPLE: str
COLORS: str
LITERAL_GROUP: str
LOCALE_GROUP: str
STRIP_RE: Incomplete
TIMEDELTA_RE: Incomplete

def is_date_format(fmt): ...
def is_timedelta_format(fmt): ...
def is_datetime(fmt): ...
def is_builtin(fmt): ...
def builtin_format_code(index): ...
def builtin_format_id(fmt): ...

class NumberFormatDescriptor(String[Incomplete]):
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...

class NumberFormat(Serialisable):
    numFmtId: Integer[Literal[False]]
    formatCode: String[Literal[False]]
    def __init__(self, numFmtId: _ConvertibleToInt, formatCode: str) -> None: ...

class NumberFormatList(Serialisable):
    # Overwritten by property below
    # count: Integer
    numFmt: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(self, count: Unused = None, numFmt=()) -> None: ...
    @property
    def count(self): ...
    def __getitem__(self, idx): ...



from _typeshed import Incomplete, Unused
from abc import abstractmethod
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.axis import ChartLines
from openpyxl.chart.label import DataLabelList
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

from ._chart import ChartBase

class _PieChartBase(ChartBase):
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Typed[DataLabelList, Literal[True]]
    dataLabels: Alias
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, varyColors: bool = True, ser=(), dLbls: DataLabelList | None = None) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class PieChart(_PieChartBase):
    tagname: str
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    firstSliceAng: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, firstSliceAng: int = 0, extLst: Unused = None, **kw) -> None: ...

class PieChart3D(_PieChartBase):
    tagname: str
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]

class DoughnutChart(_PieChartBase):
    tagname: str
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    firstSliceAng: Incomplete
    holeSize: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, firstSliceAng: int = 0, holeSize: int = 10, extLst: Unused = None, **kw) -> None: ...

class CustomSplit(Serialisable):
    tagname: str
    secondPiePt: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, secondPiePt=()) -> None: ...

class ProjectedPieChart(_PieChartBase):
    tagname: str
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    ofPieType: Incomplete
    type: Alias
    gapWidth: Incomplete
    splitType: Incomplete
    splitPos: Incomplete
    custSplit: Typed[CustomSplit, Literal[True]]
    secondPieSize: Incomplete
    serLines: Typed[ChartLines, Literal[True]]
    join_lines: Alias
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        ofPieType: str = "pie",
        gapWidth: Incomplete | None = None,
        splitType: str = "auto",
        splitPos: Incomplete | None = None,
        custSplit: CustomSplit | None = None,
        secondPieSize: int = 75,
        serLines: ChartLines | None = None,
        extLst: Unused = None,
        **kw,
    ) -> None: ...



from _typeshed import Incomplete, Unused
from abc import abstractmethod
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.layout import Layout
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText, Text
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class ChartLines(Serialisable):
    tagname: str
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    def __init__(self, spPr: GraphicalProperties | None = None) -> None: ...

class Scaling(Serialisable):
    tagname: str
    logBase: Incomplete
    orientation: Incomplete
    max: Incomplete
    min: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        logBase: Incomplete | None = None,
        orientation: str = "minMax",
        max: Incomplete | None = None,
        min: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...

class _BaseAxis(Serialisable):
    axId: Incomplete
    scaling: Typed[Scaling, Literal[False]]
    delete: Incomplete
    axPos: Incomplete
    majorGridlines: Typed[ChartLines, Literal[True]]
    minorGridlines: Typed[ChartLines, Literal[True]]
    title: Incomplete
    numFmt: Incomplete
    number_format: Alias
    majorTickMark: Incomplete
    minorTickMark: Incomplete
    tickLblPos: Incomplete
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    txPr: Typed[RichText, Literal[True]]
    textProperties: Alias
    crossAx: Incomplete
    crosses: Incomplete
    crossesAt: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        axId: Incomplete | None = None,
        scaling: Scaling | None = None,
        delete: Incomplete | None = None,
        axPos: str = "l",
        majorGridlines: ChartLines | None = None,
        minorGridlines: ChartLines | None = None,
        title: Incomplete | None = None,
        numFmt: Incomplete | None = None,
        majorTickMark: Incomplete | None = None,
        minorTickMark: Incomplete | None = None,
        tickLblPos: Incomplete | None = None,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        crossAx: Incomplete | None = None,
        crosses: Incomplete | None = None,
        crossesAt: Incomplete | None = None,
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class DisplayUnitsLabel(Serialisable):
    tagname: str
    layout: Typed[Layout, Literal[True]]
    tx: Typed[Text, Literal[True]]
    text: Alias
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    txPr: Typed[RichText, Literal[True]]
    textPropertes: Alias
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        layout: Layout | None = None,
        tx: Text | None = None,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
    ) -> None: ...

class DisplayUnitsLabelList(Serialisable):
    tagname: str
    custUnit: Incomplete
    builtInUnit: Incomplete
    dispUnitsLbl: Typed[DisplayUnitsLabel, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        custUnit: Incomplete | None = None,
        builtInUnit: Incomplete | None = None,
        dispUnitsLbl: DisplayUnitsLabel | None = None,
        extLst: Unused = None,
    ) -> None: ...

class NumericAxis(_BaseAxis):
    tagname: str
    axId: Incomplete
    scaling: Incomplete
    delete: Incomplete
    axPos: Incomplete
    majorGridlines: Incomplete
    minorGridlines: Incomplete
    title: Incomplete
    numFmt: Incomplete
    majorTickMark: Incomplete
    minorTickMark: Incomplete
    tickLblPos: Incomplete
    spPr: Incomplete
    txPr: Incomplete
    crossAx: Incomplete
    crosses: Incomplete
    crossesAt: Incomplete
    crossBetween: Incomplete
    majorUnit: Incomplete
    minorUnit: Incomplete
    dispUnits: Typed[DisplayUnitsLabelList, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        crossBetween: Incomplete | None = None,
        majorUnit: Incomplete | None = None,
        minorUnit: Incomplete | None = None,
        dispUnits: DisplayUnitsLabelList | None = None,
        extLst: Unused = None,
        **kw,
    ) -> None: ...
    @classmethod
    def from_tree(cls, node): ...

class TextAxis(_BaseAxis):
    tagname: str
    axId: Incomplete
    scaling: Incomplete
    delete: Incomplete
    axPos: Incomplete
    majorGridlines: Incomplete
    minorGridlines: Incomplete
    title: Incomplete
    numFmt: Incomplete
    majorTickMark: Incomplete
    minorTickMark: Incomplete
    tickLblPos: Incomplete
    spPr: Incomplete
    txPr: Incomplete
    crossAx: Incomplete
    crosses: Incomplete
    crossesAt: Incomplete
    auto: Incomplete
    lblAlgn: Incomplete
    lblOffset: Incomplete
    tickLblSkip: Incomplete
    tickMarkSkip: Incomplete
    noMultiLvlLbl: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        auto: Incomplete | None = None,
        lblAlgn: Incomplete | None = None,
        lblOffset: int = 100,
        tickLblSkip: Incomplete | None = None,
        tickMarkSkip: Incomplete | None = None,
        noMultiLvlLbl: Incomplete | None = None,
        extLst: Unused = None,
        **kw,
    ) -> None: ...

class DateAxis(TextAxis):
    tagname: str
    axId: Incomplete
    scaling: Incomplete
    delete: Incomplete
    axPos: Incomplete
    majorGridlines: Incomplete
    minorGridlines: Incomplete
    title: Incomplete
    numFmt: Incomplete
    majorTickMark: Incomplete
    minorTickMark: Incomplete
    tickLblPos: Incomplete
    spPr: Incomplete
    txPr: Incomplete
    crossAx: Incomplete
    crosses: Incomplete
    crossesAt: Incomplete
    auto: Incomplete
    lblOffset: Incomplete
    baseTimeUnit: Incomplete
    majorUnit: Incomplete
    majorTimeUnit: Incomplete
    minorUnit: Incomplete
    minorTimeUnit: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        auto: Incomplete | None = None,
        lblOffset: Incomplete | None = None,
        baseTimeUnit: Incomplete | None = None,
        majorUnit: Incomplete | None = None,
        majorTimeUnit: Incomplete | None = None,
        minorUnit: Incomplete | None = None,
        minorTimeUnit: Incomplete | None = None,
        extLst: Unused = None,
        **kw,
    ) -> None: ...

class SeriesAxis(_BaseAxis):
    tagname: str
    axId: Incomplete
    scaling: Incomplete
    delete: Incomplete
    axPos: Incomplete
    majorGridlines: Incomplete
    minorGridlines: Incomplete
    title: Incomplete
    numFmt: Incomplete
    majorTickMark: Incomplete
    minorTickMark: Incomplete
    tickLblPos: Incomplete
    spPr: Incomplete
    txPr: Incomplete
    crossAx: Incomplete
    crosses: Incomplete
    crossesAt: Incomplete
    tickLblSkip: Incomplete
    tickMarkSkip: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self, tickLblSkip: Incomplete | None = None, tickMarkSkip: Incomplete | None = None, extLst: Unused = None, **kw
    ) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar, overload
from typing_extensions import Literal, TypeAlias

from openpyxl.descriptors import Strict, Typed
from openpyxl.descriptors.base import Alias, Integer, MinMax, Set, _ConvertibleToFloat, _ConvertibleToInt
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

_ColorSetType: TypeAlias = Literal[
    "dk1", "lt1", "dk2", "lt2", "accent1", "accent2", "accent3", "accent4", "accent5", "accent6", "hlink", "folHlink"
]
_SystemColorVal: TypeAlias = Literal[
    "scrollBar",
    "background",
    "activeCaption",
    "inactiveCaption",
    "menu",
    "window",
    "windowFrame",
    "menuText",
    "windowText",
    "captionText",
    "activeBorder",
    "inactiveBorder",
    "appWorkspace",
    "highlight",
    "highlightText",
    "btnFace",
    "btnShadow",
    "grayText",
    "btnText",
    "inactiveCaptionText",
    "btnHighlight",
    "3dDkShadow",
    "3dLight",
    "infoText",
    "infoBk",
    "hotLight",
    "gradientActiveCaption",
    "gradientInactiveCaption",
    "menuHighlight",
    "menuBar",
]
_SchemeColorVal: TypeAlias = Literal[
    "bg1",
    "tx1",
    "bg2",
    "tx2",
    "accent1",
    "accent2",
    "accent3",
    "accent4",
    "accent5",
    "accent6",
    "hlink",
    "folHlink",
    "phClr",
    "dk1",
    "lt1",
    "dk2",
    "lt2",
]

PRESET_COLORS: Incomplete
SCHEME_COLORS: Incomplete

class Transform(Serialisable): ...

class SystemColor(Serialisable):
    tagname: str
    namespace: Incomplete
    tint: Incomplete
    shade: Incomplete
    comp: Typed[Transform, Literal[True]]
    inv: Typed[Transform, Literal[True]]
    gray: Typed[Transform, Literal[True]]
    alpha: Incomplete
    alphaOff: Incomplete
    alphaMod: Incomplete
    hue: Incomplete
    hueOff: Incomplete
    hueMod: Incomplete
    sat: Incomplete
    satOff: Incomplete
    satMod: Incomplete
    lum: Incomplete
    lumOff: Incomplete
    lumMod: Incomplete
    red: Incomplete
    redOff: Incomplete
    redMod: Incomplete
    green: Incomplete
    greenOff: Incomplete
    greenMod: Incomplete
    blue: Incomplete
    blueOff: Incomplete
    blueMod: Incomplete
    gamma: Typed[Transform, Literal[True]]
    invGamma: Typed[Transform, Literal[True]]
    val: Set[_SystemColorVal]
    lastClr: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        val: _SystemColorVal = "windowText",
        lastClr: Incomplete | None = None,
        tint: Incomplete | None = None,
        shade: Incomplete | None = None,
        comp: Transform | None = None,
        inv: Transform | None = None,
        gray: Transform | None = None,
        alpha: Incomplete | None = None,
        alphaOff: Incomplete | None = None,
        alphaMod: Incomplete | None = None,
        hue: Incomplete | None = None,
        hueOff: Incomplete | None = None,
        hueMod: Incomplete | None = None,
        sat: Incomplete | None = None,
        satOff: Incomplete | None = None,
        satMod: Incomplete | None = None,
        lum: Incomplete | None = None,
        lumOff: Incomplete | None = None,
        lumMod: Incomplete | None = None,
        red: Incomplete | None = None,
        redOff: Incomplete | None = None,
        redMod: Incomplete | None = None,
        green: Incomplete | None = None,
        greenOff: Incomplete | None = None,
        greenMod: Incomplete | None = None,
        blue: Incomplete | None = None,
        blueOff: Incomplete | None = None,
        blueMod: Incomplete | None = None,
        gamma: Transform | None = None,
        invGamma: Transform | None = None,
    ) -> None: ...

class HSLColor(Serialisable):
    tagname: str
    hue: Integer[Literal[False]]
    sat: MinMax[float, Literal[False]]
    lum: MinMax[float, Literal[False]]
    def __init__(self, hue: _ConvertibleToInt, sat: _ConvertibleToFloat, lum: _ConvertibleToFloat) -> None: ...

class RGBPercent(Serialisable):
    tagname: str
    r: MinMax[float, Literal[False]]
    g: MinMax[float, Literal[False]]
    b: MinMax[float, Literal[False]]
    def __init__(self, r: _ConvertibleToFloat, g: _ConvertibleToFloat, b: _ConvertibleToFloat) -> None: ...

_RGBPercent: TypeAlias = RGBPercent

class SchemeColor(Serialisable):
    tagname: str
    namespace: Incomplete
    tint: Incomplete
    shade: Incomplete
    comp: Incomplete
    inv: Incomplete
    gray: Incomplete
    alpha: Incomplete
    alphaOff: Incomplete
    alphaMod: Incomplete
    hue: Incomplete
    hueOff: Incomplete
    hueMod: Incomplete
    sat: Incomplete
    satOff: Incomplete
    satMod: Incomplete
    lum: Incomplete
    lumOff: Incomplete
    lumMod: Incomplete
    red: Incomplete
    redOff: Incomplete
    redMod: Incomplete
    green: Incomplete
    greenOff: Incomplete
    greenMod: Incomplete
    blue: Incomplete
    blueOff: Incomplete
    blueMod: Incomplete
    gamma: Incomplete
    invGamma: Incomplete
    val: Set[_SchemeColorVal]
    __elements__: ClassVar[tuple[str, ...]]
    @overload
    def __init__(
        self,
        tint: Incomplete | None = None,
        shade: Incomplete | None = None,
        comp: Incomplete | None = None,
        inv: Incomplete | None = None,
        gray: Incomplete | None = None,
        alpha: Incomplete | None = None,
        alphaOff: Incomplete | None = None,
        alphaMod: Incomplete | None = None,
        hue: Incomplete | None = None,
        hueOff: Incomplete | None = None,
        hueMod: Incomplete | None = None,
        sat: Incomplete | None = None,
        satOff: Incomplete | None = None,
        satMod: Incomplete | None = None,
        lum: Incomplete | None = None,
        lumOff: Incomplete | None = None,
        lumMod: Incomplete | None = None,
        red: Incomplete | None = None,
        redOff: Incomplete | None = None,
        redMod: Incomplete | None = None,
        green: Incomplete | None = None,
        greenOff: Incomplete | None = None,
        greenMod: Incomplete | None = None,
        blue: Incomplete | None = None,
        blueOff: Incomplete | None = None,
        blueMod: Incomplete | None = None,
        gamma: Incomplete | None = None,
        invGamma: Incomplete | None = None,
        *,
        val: _SchemeColorVal,
    ) -> None: ...
    @overload
    def __init__(
        self,
        tint: Incomplete | None,
        shade: Incomplete | None,
        comp: Incomplete | None,
        inv: Incomplete | None,
        gray: Incomplete | None,
        alpha: Incomplete | None,
        alphaOff: Incomplete | None,
        alphaMod: Incomplete | None,
        hue: Incomplete | None,
        hueOff: Incomplete | None,
        hueMod: Incomplete | None,
        sat: Incomplete | None,
        satOff: Incomplete | None,
        satMod: Incomplete | None,
        lum: Incomplete | None,
        lumOff: Incomplete | None,
        lumMod: Incomplete | None,
        red: Incomplete | None,
        redOff: Incomplete | None,
        redMod: Incomplete | None,
        green: Incomplete | None,
        greenOff: Incomplete | None,
        greenMod: Incomplete | None,
        blue: Incomplete | None,
        blueOff: Incomplete | None,
        blueMod: Incomplete | None,
        gamma: Incomplete | None,
        invGamma: Incomplete | None,
        val: _SchemeColorVal,
    ) -> None: ...

class ColorChoice(Serialisable):
    tagname: str
    namespace: Incomplete
    scrgbClr: Typed[_RGBPercent, Literal[True]]
    RGBPercent: Alias
    srgbClr: Incomplete
    RGB: Alias
    hslClr: Typed[HSLColor, Literal[True]]
    sysClr: Typed[SystemColor, Literal[True]]
    schemeClr: Typed[SystemColor, Literal[True]]
    prstClr: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        scrgbClr: _RGBPercent | None = None,
        srgbClr: Incomplete | None = None,
        hslClr: HSLColor | None = None,
        sysClr: SystemColor | None = None,
        schemeClr: SystemColor | None = None,
        prstClr: Incomplete | None = None,
    ) -> None: ...

_COLOR_SET: tuple[_ColorSetType, ...]

class ColorMapping(Serialisable):
    tagname: str
    bg1: Set[_ColorSetType]
    tx1: Set[_ColorSetType]
    bg2: Set[_ColorSetType]
    tx2: Set[_ColorSetType]
    accent1: Set[_ColorSetType]
    accent2: Set[_ColorSetType]
    accent3: Set[_ColorSetType]
    accent4: Set[_ColorSetType]
    accent5: Set[_ColorSetType]
    accent6: Set[_ColorSetType]
    hlink: Set[_ColorSetType]
    folHlink: Set[_ColorSetType]
    extLst: Typed[ExtensionList, Literal[True]]
    def __init__(
        self,
        bg1: str = "lt1",
        tx1: str = "dk1",
        bg2: str = "lt2",
        tx2: str = "dk2",
        accent1: str = "accent1",
        accent2: str = "accent2",
        accent3: str = "accent3",
        accent4: str = "accent4",
        accent5: str = "accent5",
        accent6: str = "accent6",
        hlink: str = "hlink",
        folHlink: str = "folHlink",
        extLst: ExtensionList | None = None,
    ) -> None: ...

class ColorChoiceDescriptor(Typed[ColorChoice, Incomplete]):
    expected_type: type[ColorChoice]
    allow_none: Literal[True]
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.descriptors.base import Typed
from openpyxl.descriptors.serialisable import Serialisable

def get_version(): ...

class DigSigBlob(Serialisable):
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]

class VectorLpstr(Serialisable):
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]

class VectorVariant(Serialisable):
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]

class ExtendedProperties(Serialisable):
    tagname: str
    Template: Incomplete
    Manager: Incomplete
    Company: Incomplete
    Pages: Incomplete
    Words: Incomplete
    Characters: Incomplete
    PresentationFormat: Incomplete
    Lines: Incomplete
    Paragraphs: Incomplete
    Slides: Incomplete
    Notes: Incomplete
    TotalTime: Incomplete
    HiddenSlides: Incomplete
    MMClips: Incomplete
    ScaleCrop: Incomplete
    HeadingPairs: Typed[VectorVariant, Literal[True]]
    TitlesOfParts: Typed[VectorLpstr, Literal[True]]
    LinksUpToDate: Incomplete
    CharactersWithSpaces: Incomplete
    SharedDoc: Incomplete
    HyperlinkBase: Incomplete
    HLinks: Typed[VectorVariant, Literal[True]]
    HyperlinksChanged: Incomplete
    DigSig: Typed[DigSigBlob, Literal[True]]
    Application: Incomplete
    AppVersion: Incomplete
    DocSecurity: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        Template: Incomplete | None = None,
        Manager: Incomplete | None = None,
        Company: Incomplete | None = None,
        Pages: Incomplete | None = None,
        Words: Incomplete | None = None,
        Characters: Incomplete | None = None,
        PresentationFormat: Incomplete | None = None,
        Lines: Incomplete | None = None,
        Paragraphs: Incomplete | None = None,
        Slides: Incomplete | None = None,
        Notes: Incomplete | None = None,
        TotalTime: Incomplete | None = None,
        HiddenSlides: Incomplete | None = None,
        MMClips: Incomplete | None = None,
        ScaleCrop: Incomplete | None = None,
        HeadingPairs: Unused = None,
        TitlesOfParts: Unused = None,
        LinksUpToDate: Incomplete | None = None,
        CharactersWithSpaces: Incomplete | None = None,
        SharedDoc: Incomplete | None = None,
        HyperlinkBase: Incomplete | None = None,
        HLinks: Unused = None,
        HyperlinksChanged: Incomplete | None = None,
        DigSig: Unused = None,
        Application: str = "Microsoft Excel",
        AppVersion: Incomplete | None = None,
        DocSecurity: Incomplete | None = None,
    ) -> None: ...
    def to_tree(self): ...



from _typeshed import Incomplete
from typing import ClassVar

from openpyxl.descriptors import DateTime
from openpyxl.descriptors.base import Alias
from openpyxl.descriptors.nested import NestedText
from openpyxl.descriptors.serialisable import Serialisable

# Does not reimplement the relevant methods, so runtime also has incompatible supertypes
class NestedDateTime(DateTime[Incomplete], NestedText):  # type: ignore[misc]
    expected_type: type[Incomplete]
    def to_tree(
        self, tagname: Incomplete | None = None, value: Incomplete | None = None, namespace: Incomplete | None = None
    ): ...

class QualifiedDateTime(NestedDateTime):
    def to_tree(
        self, tagname: Incomplete | None = None, value: Incomplete | None = None, namespace: Incomplete | None = None
    ): ...

class DocumentProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    category: Incomplete
    contentStatus: Incomplete
    keywords: Incomplete
    lastModifiedBy: Incomplete
    lastPrinted: Incomplete
    revision: Incomplete
    version: Incomplete
    last_modified_by: Alias
    subject: Incomplete
    title: Incomplete
    creator: Incomplete
    description: Incomplete
    identifier: Incomplete
    language: Incomplete
    created: Incomplete
    modified: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        category: Incomplete | None = None,
        contentStatus: Incomplete | None = None,
        keywords: Incomplete | None = None,
        lastModifiedBy: Incomplete | None = None,
        lastPrinted: Incomplete | None = None,
        revision: Incomplete | None = None,
        version: Incomplete | None = None,
        created=None,
        creator: str = "openpyxl",
        description: Incomplete | None = None,
        identifier: Incomplete | None = None,
        language: Incomplete | None = None,
        modified=None,
        subject: Incomplete | None = None,
        title: Incomplete | None = None,
    ) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar, overload
from typing_extensions import Literal

from openpyxl.chart.legend import Legend
from openpyxl.chart.pivot import PivotSource
from openpyxl.chart.plotarea import PlotArea
from openpyxl.chart.print_settings import PrintSettings
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText
from openpyxl.chart.title import Title
from openpyxl.descriptors.base import Alias, String, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.drawing.colors import ColorMapping

class ChartContainer(Serialisable):
    tagname: str
    title: Typed[Title, Literal[True]]
    autoTitleDeleted: Incomplete
    pivotFmts: Incomplete
    view3D: Incomplete
    floor: Incomplete
    sideWall: Incomplete
    backWall: Incomplete
    plotArea: Typed[PlotArea, Literal[False]]
    legend: Typed[Legend, Literal[True]]
    plotVisOnly: Incomplete
    dispBlanksAs: Incomplete
    showDLblsOverMax: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        title: Title | None = None,
        autoTitleDeleted: Incomplete | None = None,
        pivotFmts=(),
        view3D: Incomplete | None = None,
        floor: Incomplete | None = None,
        sideWall: Incomplete | None = None,
        backWall: Incomplete | None = None,
        plotArea: PlotArea | None = None,
        legend: Legend | None = None,
        plotVisOnly: bool = True,
        dispBlanksAs: str = "gap",
        showDLblsOverMax: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...

class Protection(Serialisable):
    tagname: str
    chartObject: Incomplete
    data: Incomplete
    formatting: Incomplete
    selection: Incomplete
    userInterface: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        chartObject: Incomplete | None = None,
        data: Incomplete | None = None,
        formatting: Incomplete | None = None,
        selection: Incomplete | None = None,
        userInterface: Incomplete | None = None,
    ) -> None: ...

class ExternalData(Serialisable):
    tagname: str
    autoUpdate: Incomplete
    id: String[Literal[False]]
    @overload
    def __init__(self, autoUpdate: Incomplete | None = None, *, id: str) -> None: ...
    @overload
    def __init__(self, autoUpdate: Incomplete | None, id: str) -> None: ...

class ChartSpace(Serialisable):
    tagname: str
    date1904: Incomplete
    lang: Incomplete
    roundedCorners: Incomplete
    style: Incomplete
    clrMapOvr: Typed[ColorMapping, Literal[True]]
    pivotSource: Typed[PivotSource, Literal[True]]
    protection: Typed[Protection, Literal[True]]
    chart: Typed[ChartContainer, Literal[False]]
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    txPr: Typed[RichText, Literal[True]]
    textProperties: Alias
    externalData: Typed[ExternalData, Literal[True]]
    printSettings: Typed[PrintSettings, Literal[True]]
    userShapes: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    @overload
    def __init__(
        self,
        date1904: Incomplete | None = None,
        lang: Incomplete | None = None,
        roundedCorners: Incomplete | None = None,
        style: Incomplete | None = None,
        clrMapOvr: ColorMapping | None = None,
        pivotSource: PivotSource | None = None,
        protection: Protection | None = None,
        *,
        chart: ChartContainer,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        externalData: ExternalData | None = None,
        printSettings: PrintSettings | None = None,
        userShapes: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        date1904: Incomplete | None,
        lang: Incomplete | None,
        roundedCorners: Incomplete | None,
        style: Incomplete | None,
        clrMapOvr: ColorMapping | None,
        pivotSource: PivotSource | None,
        protection: Protection | None,
        chart: ChartContainer,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        externalData: ExternalData | None = None,
        printSettings: PrintSettings | None = None,
        userShapes: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...
    def to_tree(self, tagname: Incomplete | None = None, idx: Incomplete | None = None, namespace: Incomplete | None = None): ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.picture import PictureOptions
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class Marker(Serialisable):
    tagname: str
    symbol: Incomplete
    size: Incomplete
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        symbol: Incomplete | None = None,
        size: Incomplete | None = None,
        spPr: GraphicalProperties | None = None,
        extLst: Unused = None,
    ) -> None: ...

class DataPoint(Serialisable):
    tagname: str
    idx: Incomplete
    invertIfNegative: Incomplete
    marker: Typed[Marker, Literal[True]]
    bubble3D: Incomplete
    explosion: Incomplete
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    pictureOptions: Typed[PictureOptions, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        idx: Incomplete | None = None,
        invertIfNegative: Incomplete | None = None,
        marker: Marker | None = None,
        bubble3D: Incomplete | None = None,
        explosion: Incomplete | None = None,
        spPr: GraphicalProperties | None = None,
        pictureOptions: PictureOptions | None = None,
        extLst: Unused = None,
    ) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class ManualLayout(Serialisable):
    tagname: str
    layoutTarget: Incomplete
    xMode: Incomplete
    yMode: Incomplete
    wMode: Incomplete
    hMode: Incomplete
    x: Incomplete
    y: Incomplete
    w: Incomplete
    width: Alias
    h: Incomplete
    height: Alias
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        layoutTarget: Incomplete | None = None,
        xMode: Incomplete | None = None,
        yMode: Incomplete | None = None,
        wMode: str = "factor",
        hMode: str = "factor",
        x: Incomplete | None = None,
        y: Incomplete | None = None,
        w: Incomplete | None = None,
        h: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...

class Layout(Serialisable):
    tagname: str
    manualLayout: Typed[ManualLayout, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, manualLayout: ManualLayout | None = None, extLst: Unused = None) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.data_source import AxDataSource, NumDataSource, StrRef
from openpyxl.chart.error_bar import ErrorBars
from openpyxl.chart.label import DataLabelList
from openpyxl.chart.marker import Marker
from openpyxl.chart.picture import PictureOptions
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.trendline import Trendline
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

attribute_mapping: Incomplete

class SeriesLabel(Serialisable):
    tagname: str
    strRef: Typed[StrRef, Literal[True]]
    v: Incomplete
    value: Alias
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, strRef: StrRef | None = None, v: Incomplete | None = None) -> None: ...

class Series(Serialisable):
    tagname: str
    idx: Incomplete
    order: Incomplete
    tx: Typed[SeriesLabel, Literal[True]]
    title: Alias
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Incomplete
    pictureOptions: Typed[PictureOptions, Literal[True]]
    dPt: Incomplete
    data_points: Alias
    dLbls: Typed[DataLabelList, Literal[True]]
    labels: Alias
    trendline: Typed[Trendline, Literal[True]]
    errBars: Typed[ErrorBars, Literal[True]]
    cat: Typed[AxDataSource, Literal[True]]
    identifiers: Alias
    val: Typed[NumDataSource, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    invertIfNegative: Incomplete
    shape: Incomplete
    xVal: Typed[AxDataSource, Literal[True]]
    yVal: Typed[NumDataSource, Literal[True]]
    bubbleSize: Typed[NumDataSource, Literal[True]]
    zVal: Alias
    bubble3D: Incomplete
    marker: Typed[Marker, Literal[True]]
    smooth: Incomplete
    explosion: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        idx: int = 0,
        order: int = 0,
        tx: SeriesLabel | None = None,
        spPr: GraphicalProperties | None = None,
        pictureOptions: PictureOptions | None = None,
        dPt=(),
        dLbls: DataLabelList | None = None,
        trendline: Trendline | None = None,
        errBars: ErrorBars | None = None,
        cat: AxDataSource | None = None,
        val: NumDataSource | None = None,
        invertIfNegative: Incomplete | None = None,
        shape: Incomplete | None = None,
        xVal: AxDataSource | None = None,
        yVal: NumDataSource | None = None,
        bubbleSize: NumDataSource | None = None,
        bubble3D: Incomplete | None = None,
        marker: Marker | None = None,
        smooth: Incomplete | None = None,
        explosion: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...
    def to_tree(self, tagname: Incomplete | None = None, idx: Incomplete | None = None): ...  # type: ignore[override]

class XYSeries(Series):
    idx: Incomplete
    order: Incomplete
    tx: Incomplete
    spPr: Incomplete
    dPt: Incomplete
    dLbls: Incomplete
    trendline: Incomplete
    errBars: Incomplete
    xVal: Incomplete
    yVal: Incomplete
    invertIfNegative: Incomplete
    bubbleSize: Incomplete
    bubble3D: Incomplete
    marker: Incomplete
    smooth: Incomplete



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.data_source import NumFmt
from openpyxl.chart.layout import Layout
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText, Text
from openpyxl.descriptors.base import Alias, String, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class TrendlineLabel(Serialisable):
    tagname: str
    layout: Typed[Layout, Literal[True]]
    tx: Typed[Text, Literal[True]]
    numFmt: Typed[NumFmt, Literal[True]]
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    txPr: Typed[RichText, Literal[True]]
    textProperties: Alias
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        layout: Layout | None = None,
        tx: Text | None = None,
        numFmt: NumFmt | None = None,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        extLst: Unused = None,
    ) -> None: ...

class Trendline(Serialisable):
    tagname: str
    name: String[Literal[True]]
    spPr: Typed[ExtensionList, Literal[True]]
    graphicalProperties: Alias
    trendlineType: Incomplete
    order: Incomplete
    period: Incomplete
    forward: Incomplete
    backward: Incomplete
    intercept: Incomplete
    dispRSqr: Incomplete
    dispEq: Incomplete
    trendlineLbl: Typed[ExtensionList, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        name: str | None = None,
        spPr: ExtensionList | None = None,
        trendlineType: str = "linear",
        order: Incomplete | None = None,
        period: Incomplete | None = None,
        forward: Incomplete | None = None,
        backward: Incomplete | None = None,
        intercept: Incomplete | None = None,
        dispRSqr: Incomplete | None = None,
        dispEq: Incomplete | None = None,
        trendlineLbl: ExtensionList | None = None,
        extLst: Unused = None,
    ) -> None: ...



from _typeshed import Incomplete, Unused
from abc import abstractmethod
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable as Serialisable

class _DataLabelBase(Serialisable):
    numFmt: Incomplete
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    txPr: Typed[RichText, Literal[True]]
    textProperties: Alias
    dLblPos: Incomplete
    position: Alias
    showLegendKey: Incomplete
    showVal: Incomplete
    showCatName: Incomplete
    showSerName: Incomplete
    showPercent: Incomplete
    showBubbleSize: Incomplete
    showLeaderLines: Incomplete
    separator: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        numFmt: Incomplete | None = None,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        dLblPos: Incomplete | None = None,
        showLegendKey: Incomplete | None = None,
        showVal: Incomplete | None = None,
        showCatName: Incomplete | None = None,
        showSerName: Incomplete | None = None,
        showPercent: Incomplete | None = None,
        showBubbleSize: Incomplete | None = None,
        showLeaderLines: Incomplete | None = None,
        separator: Incomplete | None = None,
        extLst: Unused = None,
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class DataLabel(_DataLabelBase):
    tagname: str
    idx: Incomplete
    numFmt: Incomplete
    spPr: Incomplete
    txPr: Incomplete
    dLblPos: Incomplete
    showLegendKey: Incomplete
    showVal: Incomplete
    showCatName: Incomplete
    showSerName: Incomplete
    showPercent: Incomplete
    showBubbleSize: Incomplete
    showLeaderLines: Incomplete
    separator: Incomplete
    extLst: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, idx: int = 0, **kw) -> None: ...

class DataLabelList(_DataLabelBase):
    tagname: str
    dLbl: Incomplete
    delete: Incomplete
    numFmt: Incomplete
    spPr: Incomplete
    txPr: Incomplete
    dLblPos: Incomplete
    showLegendKey: Incomplete
    showVal: Incomplete
    showCatName: Incomplete
    showSerName: Incomplete
    showPercent: Incomplete
    showBubbleSize: Incomplete
    showLeaderLines: Incomplete
    separator: Incomplete
    extLst: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, dLbl=(), delete: Incomplete | None = None, **kw) -> None: ...



from _typeshed import Incomplete, Unused
from abc import abstractmethod
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.axis import ChartLines, NumericAxis, _BaseAxis
from openpyxl.chart.label import DataLabelList
from openpyxl.chart.updown_bars import UpDownBars
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList

from ._chart import ChartBase

class _LineChartBase(ChartBase):
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Typed[DataLabelList, Literal[True]]
    dataLabels: Alias
    dropLines: Typed[ChartLines, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        grouping: str = "standard",
        varyColors: Incomplete | None = None,
        ser=(),
        dLbls: DataLabelList | None = None,
        dropLines: ChartLines | None = None,
        **kw,
    ) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class LineChart(_LineChartBase):
    tagname: str
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dropLines: Incomplete
    hiLowLines: Typed[ChartLines, Literal[True]]
    upDownBars: Typed[UpDownBars, Literal[True]]
    marker: Incomplete
    smooth: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    x_axis: Typed[_BaseAxis, Literal[False]]
    y_axis: Typed[NumericAxis, Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        hiLowLines: ChartLines | None = None,
        upDownBars: UpDownBars | None = None,
        marker: Incomplete | None = None,
        smooth: Incomplete | None = None,
        extLst: Unused = None,
        **kw,
    ) -> None: ...

class LineChart3D(_LineChartBase):
    tagname: str
    grouping: Incomplete
    varyColors: Incomplete
    ser: Incomplete
    dLbls: Incomplete
    dropLines: Incomplete
    gapDepth: Incomplete
    hiLowLines: Typed[ChartLines, Literal[True]]
    upDownBars: Typed[UpDownBars, Literal[True]]
    marker: Incomplete
    smooth: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    x_axis: Typed[ExtensionList, Literal[False]]
    y_axis: Typed[ExtensionList, Literal[False]]
    z_axis: Typed[ExtensionList, Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        gapDepth: Incomplete | None = None,
        hiLowLines: ChartLines | None = None,
        upDownBars: UpDownBars | None = None,
        marker: Incomplete | None = None,
        smooth: Incomplete | None = None,
        **kw,
    ) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal, TypeAlias

from openpyxl.descriptors.base import Alias, NoneSet, Typed
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.drawing.fill import GradientFillProperties, PatternFillProperties
from openpyxl.drawing.geometry import CustomGeometry2D, PresetGeometry2D, Scene3D, Shape3D, Transform2D
from openpyxl.drawing.line import LineProperties

_GraphicalPropertiesBwMode: TypeAlias = Literal[
    "clr", "auto", "gray", "ltGray", "invGray", "grayWhite", "blackGray", "blackWhite", "black", "white", "hidden"
]

class GraphicalProperties(Serialisable):
    tagname: str
    bwMode: NoneSet[_GraphicalPropertiesBwMode]
    xfrm: Typed[Transform2D, Literal[True]]
    transform: Alias
    custGeom: Typed[CustomGeometry2D, Literal[True]]
    prstGeom: Typed[PresetGeometry2D, Literal[True]]
    noFill: Incomplete
    solidFill: Incomplete
    gradFill: Typed[GradientFillProperties, Literal[True]]
    pattFill: Typed[PatternFillProperties, Literal[True]]
    ln: Typed[LineProperties, Literal[True]]
    line: Alias
    scene3d: Typed[Scene3D, Literal[True]]
    sp3d: Typed[Shape3D, Literal[True]]
    shape3D: Alias
    extLst: Typed[Incomplete, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        bwMode: _GraphicalPropertiesBwMode | Literal["none"] | None = None,
        xfrm: Transform2D | None = None,
        noFill: Incomplete | None = None,
        solidFill: Incomplete | None = None,
        gradFill: GradientFillProperties | None = None,
        pattFill: PatternFillProperties | None = None,
        ln: Incomplete | None = None,
        scene3d: Scene3D | None = None,
        custGeom: CustomGeometry2D | None = None,
        prstGeom: PresetGeometry2D | None = None,
        sp3d: Shape3D | None = None,
        extLst: Unused = None,
    ) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal, TypeAlias

from openpyxl.descriptors.base import Alias, Integer, MinMax, NoneSet, Typed, _ConvertibleToFloat, _ConvertibleToInt
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.drawing.fill import GradientFillProperties, PatternFillProperties

_LineEndPropertiesType: TypeAlias = Literal["none", "triangle", "stealth", "diamond", "oval", "arrow"]
_LineEndPropertiesWLen: TypeAlias = Literal["sm", "med", "lg"]
_LinePropertiesCap: TypeAlias = Literal["rnd", "sq", "flat"]
_LinePropertiesCmpd: TypeAlias = Literal["sng", "dbl", "thickThin", "thinThick", "tri"]
_LinePropertiesAlgn: TypeAlias = Literal["ctr", "in"]

class LineEndProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    type: NoneSet[_LineEndPropertiesType]
    w: NoneSet[_LineEndPropertiesWLen]
    len: NoneSet[_LineEndPropertiesWLen]
    def __init__(
        self,
        type: _LineEndPropertiesType | Literal["none"] | None = None,
        w: _LineEndPropertiesWLen | Literal["none"] | None = None,
        len: _LineEndPropertiesWLen | Literal["none"] | None = None,
    ) -> None: ...

class DashStop(Serialisable):
    tagname: str
    namespace: Incomplete
    d: Integer[Literal[False]]
    length: Alias
    sp: Integer[Literal[False]]
    space: Alias
    def __init__(self, d: _ConvertibleToInt = 0, sp: _ConvertibleToInt = 0) -> None: ...

class DashStopList(Serialisable):
    ds: Incomplete
    def __init__(self, ds: Incomplete | None = None) -> None: ...

class LineProperties(Serialisable):
    tagname: str
    namespace: Incomplete
    w: MinMax[float, Literal[True]]
    width: Alias
    cap: NoneSet[_LinePropertiesCap]
    cmpd: NoneSet[_LinePropertiesCmpd]
    algn: NoneSet[_LinePropertiesAlgn]
    noFill: Incomplete
    solidFill: Incomplete
    gradFill: Typed[GradientFillProperties, Literal[True]]
    pattFill: Typed[PatternFillProperties, Literal[True]]
    prstDash: Incomplete
    dashStyle: Alias
    custDash: Typed[DashStop, Literal[True]]
    round: Incomplete
    bevel: Incomplete
    miter: Incomplete
    headEnd: Typed[LineEndProperties, Literal[True]]
    tailEnd: Typed[LineEndProperties, Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        w: _ConvertibleToFloat | None = None,
        cap: _LinePropertiesCap | Literal["none"] | None = None,
        cmpd: _LinePropertiesCmpd | Literal["none"] | None = None,
        algn: _LinePropertiesAlgn | Literal["none"] | None = None,
        noFill: Incomplete | None = None,
        solidFill: Incomplete | None = None,
        gradFill: GradientFillProperties | None = None,
        pattFill: PatternFillProperties | None = None,
        prstDash: Incomplete | None = None,
        custDash: DashStop | None = None,
        round: Incomplete | None = None,
        bevel: Incomplete | None = None,
        miter: Incomplete | None = None,
        headEnd: LineEndProperties | None = None,
        tailEnd: LineEndProperties | None = None,
        extLst: Unused = None,
    ) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.layout import Layout
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.chart.text import RichText, Text
from openpyxl.descriptors import Strict, Typed
from openpyxl.descriptors.base import Alias
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class Title(Serialisable):
    tagname: str
    tx: Typed[Text, Literal[True]]
    text: Alias
    layout: Typed[Layout, Literal[True]]
    overlay: Incomplete
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    txPr: Typed[RichText, Literal[True]]
    body: Alias
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        tx: Text | None = None,
        layout: Layout | None = None,
        overlay: Incomplete | None = None,
        spPr: GraphicalProperties | None = None,
        txPr: RichText | None = None,
        extLst: Unused = None,
    ) -> None: ...

def title_maker(text): ...

class TitleDescriptor(Typed[Title, Incomplete]):
    expected_type: type[Title]
    allow_none: Literal[True]
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...



from _typeshed import Incomplete
from abc import abstractmethod
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.axis import NumericAxis, SeriesAxis, TextAxis
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

from ._3d import _3DBase
from ._chart import ChartBase

class BandFormat(Serialisable):
    tagname: str
    idx: Incomplete
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, idx: int = 0, spPr: GraphicalProperties | None = None) -> None: ...

class BandFormatList(Serialisable):
    tagname: str
    bandFmt: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, bandFmt=()) -> None: ...

class _SurfaceChartBase(ChartBase):
    wireframe: Incomplete
    ser: Incomplete
    bandFmts: Typed[BandFormatList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, wireframe: Incomplete | None = None, ser=(), bandFmts: BandFormatList | None = None, **kw) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class SurfaceChart3D(_SurfaceChartBase, _3DBase):
    tagname: str
    wireframe: Incomplete
    ser: Incomplete
    bandFmts: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    x_axis: Typed[TextAxis, Literal[False]]
    y_axis: Typed[NumericAxis, Literal[False]]
    z_axis: Typed[SeriesAxis, Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, **kw) -> None: ...

class SurfaceChart(SurfaceChart3D):
    tagname: str
    wireframe: Incomplete
    ser: Incomplete
    bandFmts: Incomplete
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(self, **kw) -> None: ...



from _typeshed import Incomplete, Unused
from typing import ClassVar
from typing_extensions import Literal

from openpyxl.chart.data_source import NumDataSource
from openpyxl.chart.shapes import GraphicalProperties
from openpyxl.descriptors.base import Alias, Typed
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.serialisable import Serialisable

class ErrorBars(Serialisable):
    tagname: str
    errDir: Incomplete
    direction: Alias
    errBarType: Incomplete
    style: Alias
    errValType: Incomplete
    size: Alias
    noEndCap: Incomplete
    plus: Typed[NumDataSource, Literal[True]]
    minus: Typed[NumDataSource, Literal[True]]
    val: Incomplete
    spPr: Typed[GraphicalProperties, Literal[True]]
    graphicalProperties: Alias
    extLst: Typed[ExtensionList, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        errDir: Incomplete | None = None,
        errBarType: str = "both",
        errValType: str = "fixedVal",
        noEndCap: Incomplete | None = None,
        plus: NumDataSource | None = None,
        minus: NumDataSource | None = None,
        val: Incomplete | None = None,
        spPr: GraphicalProperties | None = None,
        extLst: Unused = None,
    ) -> None: ...



from _typeshed import Incomplete

from openpyxl.descriptors import Strict
from openpyxl.descriptors.base import Bool, Convertible, Descriptor, Float, Integer, MinMax, NoneSet, Set, String
from openpyxl.descriptors.serialisable import Serialisable

# NOTE: # type: ignore[misc]: Class does not reimplement the relevant methods, so runtime also has incompatible supertypes

class Nested(Descriptor[Incomplete]):
    nested: bool
    attribute: str
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...
    def from_tree(self, node): ...
    def to_tree(
        self, tagname: Incomplete | None = None, value: Incomplete | None = None, namespace: Incomplete | None = None
    ): ...

class NestedValue(Nested, Convertible[Incomplete, Incomplete]): ...  # type: ignore[misc]

class NestedText(NestedValue):
    def from_tree(self, node): ...
    def to_tree(
        self, tagname: Incomplete | None = None, value: Incomplete | None = None, namespace: Incomplete | None = None
    ): ...

class NestedFloat(NestedValue, Float[Incomplete]): ...  # type: ignore[misc]
class NestedInteger(NestedValue, Integer[Incomplete]): ...  # type: ignore[misc]
class NestedString(NestedValue, String[Incomplete]): ...  # type: ignore[misc]

class NestedBool(NestedValue, Bool[Incomplete]):  # type: ignore[misc]
    def from_tree(self, node): ...

class NestedNoneSet(Nested, NoneSet[Incomplete]): ...
class NestedSet(Nested, Set[Incomplete]): ...
class NestedMinMax(Nested, MinMax[Incomplete, Incomplete]): ...  # type: ignore[misc]

class EmptyTag(Nested, Bool[Incomplete]):  # type: ignore[misc]
    def from_tree(self, node): ...
    def to_tree(
        self, tagname: Incomplete | None = None, value: Incomplete | None = None, namespace: Incomplete | None = None
    ): ...
